<?php

/**
 *
 */
function jobtrack_page_user($user, $assigned = FALSE, $state = NULL) {
  drupal_add_css(drupal_get_path('module', 'jobtrack') .'/jobtrack-tickets.css');
  if ($assigned) {
    drupal_set_title(t("@username's assigned tickets", array('@username' => $user->name)));
  }
  else {
    drupal_set_title(t("@username's latest tickets", array('@username' => $user->name)));
  }

  if (!$state) {
    $state = 'all open';
  }
  $state = _jobtrack_get_state($state);

  $rows = array();
  $header = array(
    array('data' => t('Id'), 'field' => 'n.nid', 'class' => 'id'),
    array('data' => t('Ticket'), 'field' => 'n.title'),
    array('data' => t('Updated'), 'field' => 'last_updated', 'sort' => 'desc'),
    array('data' => t('Reported by'), 'field' => 'n.uid'),
    array('data' => t('State'), 'field' => 't.state'),
    array('data' => t('Prioriy'), 'field' => 't.priority'),
    array('data' => t('Updates'), 'field' => 'l.comment_count'),
  );

  if ($state < 0) {
    $state = 'AND t.state NOT IN ('. implode(', ', _jobtrack_get_state(JOBTRACK_STATE_CLOSED)) .')';
  }
  else {
    $state = $state ? "AND t.state = $state" : '';
  }

  if (_jobtrack_access_tickets() > 1) {
    $header[] = array('data' => t('Client'), 'field' => 't.client');
  }

  $clients = jobtrack_search_available_clients();
  $sql = "SELECT DISTINCT(n.nid), n.title, n.type, n.changed, n.uid, u.name, GREATEST(n.changed, l.last_comment_timestamp) AS last_updated, l.comment_count, t.state, t.priority, t.client FROM {node} n LEFT JOIN {jobtrack_ticket} t ON n.nid = t.nid INNER JOIN {node_comment_statistics} l ON n.nid = l.nid INNER JOIN {users} u ON n.uid = u.uid LEFT JOIN {comments} c ON n.nid = c.nid WHERE (c.status = ". COMMENT_PUBLISHED ." OR c.status IS NULL) AND n.status = 1 AND n.type = 'jobtrack_ticket' AND client IN (". implode(', ', $clients) .") $state";
  if ($assigned) {
    $sql .= " AND t.assigned = $user->uid";
  }
  if (!$assigned || (user_access('only view own tickets') && !user_access('administer jobtrack') && !user_access('edit any ticket') && !user_access('delete any ticket'))) {
    $sql .= " AND n.uid = $user->uid";
  }
  $sql = db_rewrite_sql($sql);
  $sql_count = "SELECT COUNT(DISTINCT(n.nid)) FROM {node} n LEFT JOIN {jobtrack_ticket} t ON n.nid = t.nid LEFT JOIN {comments} c ON n.nid = c.nid WHERE (c.status = ". COMMENT_PUBLISHED ." OR c.status IS NULL) AND n.status = 1 AND n.type = 'jobtrack_ticket' AND t.assigned = $user->uid $state";
  $sql_count = db_rewrite_sql($sql_count);

  $sql .= tablesort_sql($header);
  $result = pager_query($sql, 50, 0, $sql_count);
  $row = 0;
  $rows = array();
  while ($ticket = db_fetch_object($result)) {
    $account = user_load(array('uid' => $ticket->uid));
    $comments = l($ticket->comment_count, "node/$ticket->nid", array('fragment' => 'comments'));
    if ($new = comment_num_new($ticket->nid)) {
      $node = node_load($ticket->nid);
      $comments .= '&nbsp;('. l(format_plural($new, '1 new', '@count new'), "node/$ticket->nid", array('query' => comment_new_page_count($node->comment_count, $new, $node), 'fragment' => 'new')) .')';
    }
    $client = jobtrack_client_load($ticket->client);
    $rows[] = array('data' => array(
      array('data' => l($ticket->nid, "node/$ticket->nid"), 'class' => 'ticket-id'),
      array('data' => l(_jobtrack_truncate($ticket->title), "node/$ticket->nid"), 'class' => 'ticket-title'),
      array('data' => format_date($ticket->changed, 'small'), 'class' => 'ticket-updated'),
      array('data' => l(_jobtrack_truncate($account->name, 24), "user/$account->uid"), 'class' => 'ticket-reported'),
      array('data' => _jobtrack_state($ticket->state), 'class' => 'ticket-state'),
      array('data' => _jobtrack_priorities($ticket->priority), 'class' => 'ticket-priority'),
      array('data' => $comments, 'class' => 'ticket-updates'),
    ), 'class' => "state-$ticket->state priority-$ticket->priority");
    if (_jobtrack_access_tickets() > 1) {
      $rows[$row]['data'][] = l($client->name, "jobtrack/$client->path");
    }
    $row++;
  }
  return theme('table', $header, $rows, array('class' => 'jobtrack')) . theme('pager');
}
